@@ -16,6 +16,7 @@ from outflow.core.logging import logger, set_plugins_loggers_config
 from outflow.core.pipeline import config, context, settings
 from outflow.core.pipeline.context_manager import PipelineContextManager
 from outflow.core.plugin import Plugin
+from outflow.core.workflow import WorkflowManager
 
 
 class Pipeline:
@@ -61,13 +62,6 @@ class Pipeline:
             ["-L", "--local-mode"],
             {"help": "Force local mode", "action": "store_true"},
         ),
-        # "static_typecheck": (
-        #     ["--static-typecheck"],
-        #     {
-        #         "help": "Use the static typechecker backend to validate the given command",
-        #         "action": "store_true",
-        #     },
-        # ),
         "backend": (
             ["--backend", "-b"],
             {
@@ -89,6 +83,7 @@ class Pipeline:
     )
 
     def __init__(self, **kwargs):
+        self.result = []
         self._kwargs = kwargs
 
     def _init(
@@ -152,12 +147,13 @@ class Pipeline:
         if system_args.backend:
             pipeline_config["backend"] = system_args.backend
 
-        # if system_args.static_typecheck:
-        #     pipeline_config["backend"] = "static_typechecker"
-
         self.load_plugins()
         self.load_backends()
 
+    def _dispose(self):
+        # close database connections, clear Model metadata and bubble up the exception if any
+        context.databases.close()
+
     def load_plugins(self):
         from outflow.core.pipeline import context, settings
 
@@ -245,20 +241,20 @@ class Pipeline:
             print(f"Outflow version '{outflow_version}'")
             return 0
 
-        # from outflow.core.pipeline import context
-
         context.force_dry_run = self.force_dry_run
 
-        backend = self.setup_backend()
-
         command = self.root_command.parse_command_tree(self.command_argv)
+
+        # if no argument is supplied, the help message is triggered
+        # and command tree parser returns 0
+        if command == 0:
+            return command
+
         context.db_untracked = command.db_untracked
 
-        if (
-            command.backend is not None and command.backend != backend.name
-        ):  # FIXME find a way to start ray before parsing command
-            backend.clean()
-            backend = self.setup_backend(command.backend)
+        backend = self.setup_backend(command.backend)
+
+        context.backend_name = backend.name
 
         if command.db_untracked:
             logger.debug(
@@ -268,10 +264,17 @@ class Pipeline:
         db_config = self.create_config_in_db()
         db_run = self.create_run_in_db(db_config)
 
+        return_code = 1
+
         try:
             db_run.start()
-            self.result = backend.run(task_list=command._setup_tasks())
+            with WorkflowManager():
+                task_list = command._setup_tasks()
+            if task_list:
+                backend.check_task_io(task_list)
+                self.result = backend.run(task_list=task_list)
             db_run.success()
+            return_code = 0
         except Exception as e:
             db_run.fail()
             from outflow.management.models.runtime_exception import RuntimeException
@@ -284,13 +287,12 @@ class Pipeline:
                 time=datetime.datetime.now(),
             )
 
+            # logger.print_exception() # TODO add this for file_handler but find a way to not duplicate exception in terminal
             raise
         finally:
-            # close database connections, clear Model metadata and bubble up the exception if any
-            context.databases.close()
             backend.clean()
 
-        return 0
+        return return_code
 
     @staticmethod
     def create_config_in_db():
@@ -337,6 +339,7 @@ class Pipeline:
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Exit context, resetting parent_context and context to None"""
+        self._dispose()
         self.pipeline_context_manager.__exit__(exc_type, exc_val, exc_tb)
 
 
