@@ -155,6 +155,9 @@ class Plugin:
                 inspect.getmembers(sys.modules[submodule_name], is_model)
             )
 
+        if context._models is None:
+            context._models = []
+
         # return model classes in the pipeline context
         for name, model in set(model_classes):
             context._models.append(model)
