@@ -2,13 +2,14 @@
 from copy import deepcopy
 from uuid import uuid4
 
-from outflow.core.exceptions import TaskException
+from outflow.core.exceptions import TaskException, IOCheckerError
 from outflow.core.generic.string import to_camel_case
 from outflow.core.pipeline import context, config
-from outflow.core.workflow import Workflow
+from outflow.core.workflow import WorkflowManager
 from outflow.core.logging import logger
 
 from .metaclass import TaskMeta
+from ..target import NoDefault
 
 
 class BaseTask(metaclass=TaskMeta):
@@ -27,12 +28,13 @@ class BaseTask(metaclass=TaskMeta):
     def __init__(self, *args, **kwargs):
         self.uuid = str(uuid4())
         self.bind_kwargs = kwargs
-        self.workflow = Workflow(self)
+        # self.workflow = Workflow(self)
+        self.workflow = WorkflowManager.get_context()["workflow"]
+        self.workflow.add_node(self)
         self.terminating = False
-        self.parallel_workflow_id = None
-        self.parallel_workflow_nb = None
         self.num_cpus = 1
         self.db_task = self.create_task_in_db()
+        self._skip = False
 
     def __repr__(self):
         return self.name
@@ -131,11 +133,13 @@ class BaseTask(metaclass=TaskMeta):
         Example: task_or_list = self << task_or_list
         """
 
-        if isinstance(task_or_list, list):
+        try:
+            iter(task_or_list)
+        except TypeError:
+            self.add_parent(task_or_list)
+        else:
             for task in task_or_list:
                 self << task
-        else:
-            self.add_parent(task_or_list)
 
         return task_or_list
 
@@ -145,40 +149,33 @@ class BaseTask(metaclass=TaskMeta):
 
         Example: task_or_list = self >> task_or_list
         """
-        if isinstance(task_or_list, list):
+        try:
+            iter(task_or_list)
+        except TypeError:
+            task_or_list.add_parent(self)
+        else:
             for task in task_or_list:
                 self >> task
-        else:
-            task_or_list.add_parent(self)
 
         return task_or_list
 
     @staticmethod
-    def add_edge(parent_task, child_task):
-        if parent_task.workflow is None and child_task.workflow is None:
-            workflow = Workflow(parent_task)
-            workflow.add_node(child_task)
-        elif parent_task.workflow is None:
-            workflow = child_task.workflow
-            workflow.add_node(parent_task)
-        elif child_task.workflow is None:
-            workflow = parent_task.workflow
-            workflow.add_node(child_task)
-        elif parent_task.workflow is not child_task.workflow:
-            parent_task.workflow.merge(child_task.workflow)
-            workflow = parent_task.workflow
+    def add_edge(parent_task, child_task, save_in_db=True):
+        if save_in_db:
+            BaseTask.add_edge_in_db(parent_task, child_task)  # TODO
+
+        if parent_task.workflow == child_task.workflow:
+            parent_task.workflow.add_edge(parent_task, child_task)
         else:
-            workflow = parent_task.workflow
+            workflow = child_task.workflow
+            workflow.add_external_edge(parent_task, child_task)
+            BaseTask.add_edge(parent_task, workflow.manager_task, save_in_db=False)
 
-        BaseTask.add_edge_in_db(parent_task, child_task)
+    def add_child(self, child_task, save_in_db=True):
+        self.add_edge(self, child_task, save_in_db)
 
-        workflow.add_edge(parent_task, child_task)
-
-    def add_child(self, child_task):
-        self.add_edge(self, child_task)
-
-    def add_parent(self, parent_task):
-        self.add_edge(parent_task, self)
+    def add_parent(self, parent_task, save_in_db=True):
+        self.add_edge(parent_task, self, save_in_db)
 
     @property
     def parents(self):
@@ -215,6 +212,7 @@ class BaseTask(metaclass=TaskMeta):
         return kwargs
 
     def __call__(self, *args, **kwargs):
+        logger.debug(f"Running task {self.name}")
         if args:
             raise Exception(
                 "Task use keyword-only arguments but positional arguments were passed to the run function"
@@ -223,24 +221,53 @@ class BaseTask(metaclass=TaskMeta):
     def bind(self, **kwargs):
         self.bind_kwargs.update(kwargs)
 
-    def copy(self):
-        task_copy = deepcopy(self)
+    def __deepcopy__(self, memodict={}):
+
+        cls = self.__class__
+        task_copy = cls.__new__(cls)
+        memodict[id(self)] = task_copy
+        for k, v in self.__dict__.items():
+            if k not in ["uuid", "workflow", "db_task", "external_edges"]:
+                setattr(task_copy, k, deepcopy(v, memodict))
         task_copy.uuid = uuid4()
+        task_copy.workflow = self.workflow
         return task_copy
 
-    def recursive_set_skip(self, task):
-        task.skip = True
-        for child_task in task.children:
-            self.recursive_set_skip(child_task)
+    @property
+    def skip(self):
+        return self._skip
+
+    @skip.setter
+    def skip(self, val: bool):
+        if val:
+            self.db_task.skip()
+        self._skip = val
+
+        if hasattr(self, "inner_workflow"):
+            for inner_task in self.inner_workflow:
+                inner_task.skip = val
 
     def create_task_in_db(self):
         from outflow.management.models.task import Task as TaskModel
 
+        json_inputs, json_outputs = (
+            {
+                target.name: {
+                    "type": getattr(target.type, "__name__", None) or repr(target.type),
+                    "default": repr(target.default),
+                }
+                for target in io
+            }
+            for io in [self.inputs.values(), self.outputs.values()]
+        )
         task_row = TaskModel.create(
             plugin=self.plugin_name,
             name=self.name,
             run=context.db_run,
             uuid=self.uuid,
+            input_targets=json_inputs,
+            output_targets=json_outputs,
+            workflow=self.workflow.db_workflow,
         )
 
         return task_row
@@ -253,3 +280,15 @@ class BaseTask(metaclass=TaskMeta):
         logger.debug(f"adding edge {child_task} >> {parent_task.name}")
         parent_task.db_task.downstream_tasks.append(child_task.db_task)
         context.session.commit()
+
+    def check_inputs(self, task_inputs):
+        for target_name, target in self.inputs.items():
+            if (
+                target_name not in task_inputs
+                and self.inputs[target_name].default == NoDefault
+                and target_name not in self.bind_kwargs
+            ):
+                raise IOCheckerError(
+                    f"Task {self.name} did not get all expected inputs: expected {[k for k in self.inputs.keys()]}, got "
+                    f"{[k for k in task_inputs.keys()]}"
+                )
