@@ -4,7 +4,7 @@
 from outflow.core.db import Model
 from outflow.management.models.mixins import Executable
 from outflow.management.models.run import Run
-from sqlalchemy import Column, ForeignKey, Integer, String
+from sqlalchemy import Column, ForeignKey, Integer, String, JSON
 from sqlalchemy.orm import relationship
 
 
@@ -18,6 +18,8 @@ class Task(Model, Executable):
     name = Column(String(256), nullable=False)
     run_id = Column(Integer, ForeignKey(Run.id), nullable=False)
     run = relationship("Run")
+    input_targets = Column(JSON, nullable=False, server_default="{}")
+    output_targets = Column(JSON, nullable=False, server_default="{}")
     upstream_tasks = relationship(
         "Task",
         secondary="edge",
@@ -25,6 +27,10 @@ class Task(Model, Executable):
         secondaryjoin="Task.id == Edge.upstream_task_id",
         backref="downstream_tasks",
     )
+    workflow_id = Column(
+        Integer, ForeignKey("workflow.id"), back_populates="tasks", nullable=False
+    )
+    workflow = relationship("Workflow", foreign_keys=[workflow_id])
 
 
 class Edge(Model):
